import WebSocket from 'ws';
import log from 'rainbowlog';

let socket;
let isConnectted = false;
const server = 'ws://172.17.7.72:7788';
// const server = 'ws://mobaai.smartstudy.com:7788';
const config = {
  gameId: '99v10',
  token: 'j3fKWoO6LRp1x53B',  // 99
  heros: ['warrior', 'warrior', 'warrior', 'warrior', 'warrior'] //warrior, shooter
};

// 发送消息
const emit = (type, params) => {
  socket.send(JSON.stringify(Object.assign({}, { type }, params)));
};

// 计算两点之间距离
const getDistance = (p1, p2) => (
  Math.sqrt(Math.pow(p1.x - p2.x, 2) + Math.pow(p1.y - p2.y, 2))
);

// 状态更新
const update = (myCamp, state) => {
  const { towers, heros } = state;
  const heroCmds = {};

  // 优先攻击靠近的英雄
  heros.forEach((hero) => {
    if (hero.camp !== myCamp || heroCmds[hero.id]) {
      return;
    }

    // 计算离每个英雄最近的英雄
    let closeHero;
    let minDistance;
    heros.forEach((opHero) => {
      if (opHero.camp === myCamp) {
        return;
      }

      const distance = getDistance(hero.position, opHero.position);
      if (
        distance < 400 &&
        opHero.status !== 'dead' &&
        (!closeHero || distance < minDistance)
      ) {
        closeHero = opHero;
        minDistance = distance;
      }
    });

    // 攻击英雄
    if (closeHero) {
      heroCmds[hero.id] = {
        // 优先使用技能
        type: hero.fireCD === 0 ? 'fire' : 'attack',
        params: {
          heroId: hero.id,
          targetType: 'hero',
          targetId: closeHero.id
        }
      };
    }
  });

  // 其次占塔
  heros.forEach((hero) => {
    if (hero.camp !== myCamp || heroCmds[hero.id]) {
      return;
    }

    // 计算离每个英雄最近的塔
    let closeTower;
    let minDistance;
    towers.forEach((tower) => {
      if (tower.camp === myCamp) {
        return;
      }

      const distance = getDistance(hero.position, tower.position);
      if (!closeTower || distance < minDistance) {
        closeTower = tower;
        minDistance = distance;
      }
    });

    // 攻击塔
    if (closeTower) {
      heroCmds[hero.id] = {
        type: 'attack',
        params: {
          heroId: hero.id,
          targetType: 'tower',
          targetId: closeTower.id,
        },
      };
    }
  });

  // 传递命令给服务器
  heros.forEach((hero) => {
    if (hero.camp !== myCamp) {
      return;
    }

    if (heroCmds[hero.id]) {
      emit(heroCmds[hero.id].type, heroCmds[hero.id].params);
    } else {
      // 如果没有可执行的命令，则攻击敌方未死的英雄
      heros.forEach((opHero) => {
        if (opHero.camp !== myCamp && opHero.status !== 'dead') {
          emit('attack', {
            heroId: hero.id,
            targetType: 'hero',
            targetId: opHero.id,
          });
        }
      });
    }
  });
};

// 创建websocket
const connect = () => {
  isConnectted = true;
  socket = new WebSocket(server);

  // 连接成功
  socket.on('open', () => {
    log.debug(`Join Home ${config.gameId}...`);
    emit('join', { gameId: config.gameId, token: config.token });
    log.debug(`Joined Home ${config.gameId}, Legend is ready!`);
  });

  // 接收消息
  socket.on('message', (message) => {
    const params = JSON.parse(message);

    // 选择英雄
    if (params.type === 'picking') {
      log.debug(`Picking Heroes: ${config.heros.join(', ')}.`);
      emit(
        'pickHero',
        { heros: config.heros },
      );
    }

    // 状态更新
    if (params.type === 'update') {
      update(params.myCamp, params.state);
    }

    // 命令错误
    if (params.type === 'cmdError') {
      log.error('CMD ERROR: ', params.message);
    }
  });

  // 错误
  socket.on('error', err => {
    log.error('socket error', err);
    isConnectted = false;
  });

  // 断开连接
  socket.on('close', () => {
    log.debug('close socket connect');
    isConnectted = false;
  });
};
connect();

// 健康检查
setInterval(() => {
  if (!isConnectted) {
    connect();
  }
}, 20);
